import { select, scaleLinear, axisBottom, axisLeft, scaleOrdinal, schemeDark2, extent } from 'd3';
import { axes } from './axes';

export const viz = (
  svg,
  {
    data,
    xValue,
    xAxisLabelText,
    xAxisLabelOffset,
    yValue,
    yAxisLabelText,
    yAxisLabelOffset,
    marginLeft,
    marginTop,
    marginRight,
    marginBottom,
    width,
    height,
    innerRectFill,
    circleRadius,
    circleOpacity,
  },
) => {
  svg.selectAll('.axis, .axis-label, .legend').remove();
  //Temporary SVG for measuring and subtracting legend width, is there a better way to do this?
  const tempSvg = select(document.body).append('svg').style('visibility', 'hidden');
  const colorScale = scaleOrdinal(schemeDark2).domain(data.map(d => d.Generation));
  const tempLegend = tempSvg.append('g');
  colorScale.domain().forEach((gen, index) => {
    const legendRow = tempLegend.append('g')
      .attr('transform', `translate(0, ${index * 20})`);
    legendRow.append('circle')
      .attr('cx', 0)
      .attr('cy', 0)
      .attr('r', 5)
      .attr('fill', colorScale(gen));
    legendRow.append('text')
      .attr('x', 15)
      .attr('y', 5)
      .text(`Generation ${gen}`);
  });
  const legendWidth = tempLegend.node().getBBox().width;
  marginRight += legendWidth + 20;
  tempSvg.remove();

  //Scales
  const xScale = scaleLinear().domain(extent(data, xValue)).range([marginLeft, width - marginRight]);
  const yScale = scaleLinear().domain(extent(data, yValue)).range([height - marginBottom, marginTop]);

  //Inner rectangle
  svg.selectAll('rect').data([null]).join('rect')
    .attr('x', marginLeft)
    .attr('y', marginTop)
    .attr('width', width - marginLeft - marginRight)
    .attr('height', height - marginTop - marginBottom)
    .attr('fill', innerRectFill);

  //X-axis
  svg.append('g')
    .attr('class', 'axis')
    .attr('transform', `translate(0,${height - marginBottom})`)
    .call(axisBottom(xScale))
    .append('text')
    .attr('class', 'axis-label')
    .attr('y', xAxisLabelOffset)
    .attr('x', width / 2)
    .attr('text-anchor', 'middle')
    .text(xAxisLabelText);

  //Y-axis
  svg.append('g')
    .attr('class', 'axis')
    .attr('transform', `translate(${marginLeft},0)`)
    .call(axisLeft(yScale))
    .append('text')
    .attr('class', 'axis-label')
    .attr('transform', 'rotate(-90)')
    .attr('y', -yAxisLabelOffset)
    .attr('x', -height / 2)
    .attr('text-anchor', 'middle')
    .text(yAxisLabelText);

  //Circles for data points
  svg.selectAll('circle').data(data).join('circle')
    .attr('cx', d => xScale(xValue(d)))
    .attr('cy', d => yScale(yValue(d)))
    .attr('r', circleRadius)
    .attr('fill', d => colorScale(d.Generation))
    .attr('opacity', circleOpacity);

  //Legend
  const legend = svg.append('g')
    .attr('class', 'legend')
    .attr('transform', `translate(${width - marginRight}, ${marginTop})`);
  colorScale.domain().forEach((gen, index) => {
    const legendRow = legend.append('g')
      .attr('transform', `translate(0, ${index * 20})`);
    legendRow.append('circle')
      .attr('cx', 15)
      .attr('cy', 10)
      .attr('r', 5)
      .attr('fill', colorScale(gen));
    legendRow.append('text')
      .attr('x', 25)
      .attr('y', 15)
      .text(`Generation ${gen}`);
  });

  axes(svg, {
    width,
    height,
    xScale,
    xAxisLabelText,
    xAxisLabelOffset,
    yScale,
    yAxisLabelText,
    yAxisLabelOffset,
    marginLeft,
    marginBottom,
  });
};